#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2023, Sabari Jaganathan <sajagana@cisco.com>
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "certified"}

DOCUMENTATION = r"""
---
module: aci_fabric_interface_policy_group
short_description: Manage Fabric Interface Policy Groups (fabric:LePortPGrp and fabric:SpPortPGrp)
description:
- Manage Fabric Interface Policy Groups on Cisco ACI fabrics.
options:
  name:
    description:
    - The name of the Fabric Leaf or Spine Interface Policy Group.
    type: str
    aliases: [ policy_group ]
  description:
    description:
    - The description of the Fabric Leaf or Spine Interface Policy Group.
    type: str
    aliases: [ descr ]
  type:
    description:
    - The type of the Fabric Leaf or Spine Interface Policy Group.
    - Use C(leaf) to create a Fabric Leaf Interface Policy Group.
    - Use C(spine) to create a Fabric Spine Interface Policy Group.
    type: str
    aliases: [ policy_group_type ]
    choices: [ leaf, spine ]
    required: true
  dwdm_policy:
    description:
    - The name of the DWDM policy to bind to the Fabric Leaf or Spine Interface Policy Group.
    type: str
  link_level_policy:
    description:
    - The name of the Link Level policy to bind to the Fabric Leaf or Spine Interface Policy Group.
    type: str
  link_flap_policy:
    description:
    - The name of the Link Flap policy to bind to the Fabric Leaf or Spine Interface Policy Group.
    type: str
  l3_interface_policy:
    description:
    - The name of the L3 Interface policy to bind to the Fabric Leaf or Spine Interface Policy Group.
    type: str
  macsec_policy:
    description:
    - The name of the MACSec policy to bind to the Fabric Leaf or Spine Interface Policy Group.
    type: str
  monitoring_policy:
    description:
    - The name of the Monitoring policy to bind to the Fabric Leaf or Spine Interface Policy Group.
    type: str
  transceiver_policy_tdn:
    description:
    - The target Dn of the Transceiver policy to bind to the Fabric Leaf or Spine Interface Policy Group.
    - The Transceiver policy group is only compatible with ACI versions 6.0(2h) and higher.
    type: str
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
  name_alias:
    description:
    - The alias for the current object. This relates to the nameAlias field in ACI.
    type: str
extends_documentation_fragment:
- cisco.aci.aci
- cisco.aci.annotation
- cisco.aci.owner

seealso:
- name: APIC Management Information Model reference
  description: More information about the internal APIC class B(fabric:LePortPGrp, fabric:SpPortPGrp).
  link: https://developer.cisco.com/docs/apic-mim-ref/
author:
- Sabari Jaganathan (@sajagana)
"""

EXAMPLES = r"""
- name: Add a Fabric Leaf Policy Group
  cisco.aci.aci_fabric_interface_policy_group:
    host: apic
    username: admin
    password: SomeSecretPassword
    name: leaf_policy_group
    type: leaf
    state: present
  delegate_to: localhost

- name: Query a Fabric Leaf Policy Group with name
  cisco.aci.aci_fabric_interface_policy_group:
    host: apic
    username: admin
    password: SomeSecretPassword
    name: leaf_policy_group
    type: leaf
    state: query
  delegate_to: localhost
  register: query_result

- name: Query all Fabric Leaf Policy Groups
  cisco.aci.aci_fabric_interface_policy_group:
    host: apic
    username: admin
    password: SomeSecretPassword
    type: leaf
    state: query
  delegate_to: localhost
  register: query_result

- name: Remove a Fabric Leaf Policy Group
  cisco.aci.aci_fabric_interface_policy_group:
    host: apic
    username: admin
    password: SomeSecretPassword
    name: leaf_policy_group
    type: leaf
    state: absent
  delegate_to: localhost
"""

RETURN = r"""
current:
  description: The existing configuration from the APIC after the module has finished
  returned: success
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production environment",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
error:
  description: The error information as returned from the APIC
  returned: failure
  type: dict
  sample:
    {
        "code": "122",
        "text": "unknown managed object class foo"
    }
raw:
  description: The raw output returned by the APIC REST API (xml or json)
  returned: parse error
  type: str
  sample: '<?xml version="1.0" encoding="UTF-8"?><imdata totalCount="1"><error code="122" text="unknown managed object class foo"/></imdata>'
sent:
  description: The actual/minimal configuration pushed to the APIC
  returned: info
  type: list
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment"
            }
        }
    }
previous:
  description: The original configuration from the APIC before the module has started
  returned: info
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
proposed:
  description: The assembled configuration from the user-provided parameters
  returned: info
  type: dict
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment",
                "name": "production"
            }
        }
    }
filter_string:
  description: The filter string used for the request
  returned: failure or debug
  type: str
  sample: ?rsp-prop-include=config-only
method:
  description: The HTTP method used for the request to the APIC
  returned: failure or debug
  type: str
  sample: POST
response:
  description: The HTTP response from the APIC
  returned: failure or debug
  type: str
  sample: OK (30 bytes)
status:
  description: The HTTP status from the APIC
  returned: failure or debug
  type: int
  sample: 200
url:
  description: The HTTP url used for the request to the APIC
  returned: failure or debug
  type: str
  sample: https://10.11.12.13/api/mo/uni/tn-production.json
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.aci.plugins.module_utils.aci import ACIModule, aci_argument_spec, aci_annotation_spec, aci_owner_spec


def main():
    argument_spec = aci_argument_spec()
    argument_spec.update(aci_annotation_spec())
    argument_spec.update(aci_owner_spec())
    argument_spec.update(
        name=dict(type="str", aliases=["policy_group"]),
        description=dict(type="str", aliases=["descr"]),
        name_alias=dict(type="str"),
        type=dict(type="str", aliases=["policy_group_type"], choices=["leaf", "spine"], required=True),
        dwdm_policy=dict(type="str"),
        link_level_policy=dict(type="str"),
        link_flap_policy=dict(type="str"),
        l3_interface_policy=dict(type="str"),
        macsec_policy=dict(type="str"),
        monitoring_policy=dict(type="str"),
        transceiver_policy_tdn=dict(type="str"),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["name"]],
            ["state", "present", ["name"]],
        ],
    )

    aci = ACIModule(module)

    name = module.params.get("name")
    description = module.params.get("description")
    state = module.params.get("state")
    name_alias = module.params.get("name_alias")
    policy_group_type = module.params.get("type")
    dwdm_policy = module.params.get("dwdm_policy")
    link_level_policy = module.params.get("link_level_policy")
    link_flap_policy = module.params.get("link_flap_policy")
    l3_interface_policy = module.params.get("l3_interface_policy")
    macsec_policy = module.params.get("macsec_policy")
    monitoring_policy = module.params.get("monitoring_policy")
    transceiver_policy_tdn = module.params.get("transceiver_policy_tdn")

    if policy_group_type == "leaf":
        policy_group_class_name = "fabricLePortPGrp"
        policy_group_class_rn = "leportgrp-{0}".format(name)
    else:
        policy_group_class_name = "fabricSpPortPGrp"
        policy_group_class_rn = "spportgrp-{0}".format(name)

    child_classes = [
        "fabricRsDwdmFabIfPol",
        "fabricRsFIfPol",
        "fabricRsFLinkFlapPol",
        "fabricRsL3IfPol",
        "fabricRsMacsecFabIfPol",
        "fabricRsMonIfFabricPol",
    ]

    if transceiver_policy_tdn is not None:
        child_classes.append("fabricRsOpticsFabIfPol")

    aci.construct_url(
        root_class=dict(
            aci_class="fabric",
            aci_rn="fabric",
        ),
        subclass_1=dict(
            aci_class="fabricFuncP",
            aci_rn="funcprof",
        ),
        subclass_2=dict(
            aci_class=policy_group_class_name,
            aci_rn=policy_group_class_rn,
            module_object=name,
            target_filter={"name": name},
        ),
        child_classes=child_classes,
    )

    aci.get_existing()

    if state == "present":
        child_configs = []
        if dwdm_policy is not None:
            child_configs.append(dict(fabricRsDwdmFabIfPol=dict(attributes=dict(tnDwdmFabIfPolName=dwdm_policy))))
        if link_level_policy is not None:
            child_configs.append(dict(fabricRsFIfPol=dict(attributes=dict(tnFabricFIfPolName=link_level_policy))))
        if link_flap_policy is not None:
            child_configs.append(dict(fabricRsFLinkFlapPol=dict(attributes=dict(tnFabricFLinkFlapPolName=link_flap_policy))))
        if l3_interface_policy is not None:
            child_configs.append(dict(fabricRsL3IfPol=dict(attributes=dict(tnL3IfPolName=l3_interface_policy))))
        if macsec_policy is not None:
            child_configs.append(dict(fabricRsMacsecFabIfPol=dict(attributes=dict(tnMacsecFabIfPolName=macsec_policy))))
        if monitoring_policy is not None:
            child_configs.append(dict(fabricRsMonIfFabricPol=dict(attributes=dict(tnMonFabricPolName=monitoring_policy))))
        if transceiver_policy_tdn is not None:
            child_configs.append(dict(fabricRsOpticsFabIfPol=dict(attributes=dict(tDn=transceiver_policy_tdn))))

        aci.payload(
            aci_class=policy_group_class_name,
            class_config=dict(
                name=name,
                descr=description,
                nameAlias=name_alias,
            ),
            child_configs=child_configs,
        )

        aci.get_diff(aci_class=policy_group_class_name)

        aci.post_config()

    if state == "absent":
        aci.delete_config()

    aci.exit_json()


if __name__ == "__main__":
    main()



#
# -*- coding: utf-8 -*-
# Copyright 2019 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
"""
The iosxr static_routes fact class
It is in this file the configuration is collected from the device
for a given resource, parsed, and the facts tree is populated
based on the configuration.
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type


import re

from copy import deepcopy

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common import utils

from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.argspec.static_routes.static_routes import (
    Static_routesArgs,
)


class Static_routesFacts(object):
    """The iosxr static_routes fact class"""

    def __init__(self, module, subspec="config", options="options"):
        self._module = module
        self.argument_spec = Static_routesArgs.argument_spec
        spec = deepcopy(self.argument_spec)
        if subspec:
            if options:
                facts_argument_spec = spec[subspec][options]
            else:
                facts_argument_spec = spec[subspec]
        else:
            facts_argument_spec = spec

        self.generated_spec = utils.generate_dict(facts_argument_spec)

    def get_device_data(self, connection):
        return connection.get_config(flags="router static")

    def populate_facts(self, connection, ansible_facts, data=None):
        """Populate the facts for static_routes
        :param connection: the device connection
        :param ansible_facts: Facts dictionary
        :param data: previously collected conf
        :rtype: dictionary
        :returns: facts
        """
        if not data:
            data = self.get_device_data(connection)

        objs = []

        if "No such configuration" not in data:
            for entry in re.compile(r"(\s) vrf").split(data):
                obj = self.render_config(self.generated_spec, entry)
                if obj:
                    objs.append(obj)

        ansible_facts["ansible_network_resources"].pop("static_routes", None)
        facts = {}

        facts["static_routes"] = []
        params = utils.validate_config(self.argument_spec, {"config": objs})
        for cfg in params["config"]:
            facts["static_routes"].append(utils.remove_empties(cfg))

        ansible_facts["ansible_network_resources"].update(facts)
        return ansible_facts

    def render_config(self, spec, conf):
        """
        Render config as dictionary structure and delete keys
          from spec for null values

        :param spec: The facts tree, generated from the argspec
        :param conf: The configuration
        :rtype: dictionary
        :returns: The generated config
        """
        config = deepcopy(spec)
        entry_list = conf.split(" address-family")
        config["address_families"] = []

        if "router static" not in entry_list[0]:
            config["vrf"] = entry_list[0].replace("!", "").strip()

        for item in entry_list[1:]:
            routes = []
            address_family = {"routes": []}
            address_family["afi"], address_family["safi"] = self.parse_af(item)

            destinations = re.findall(r"((?:\S+)/(?:\d+)) (?:.*)", item, re.M)
            for dest in set(destinations):
                route = {"next_hops": []}
                route["dest"] = dest

                regex = r"%s .+$" % dest
                cfg = re.findall(regex, item, re.M)

                for route_entry in cfg:
                    exit_point = {}
                    exit_point["forward_router_address"] = self.parse_faddr(
                        route_entry,
                    )
                    exit_point["interface"] = self.parse_intf(route_entry)
                    exit_point["admin_distance"] = self.parse_admin_distance(
                        route_entry,
                    )

                    for x in [
                        "tag",
                        "tunnel-id",
                        "metric",
                        "description",
                        "track",
                        "vrflabel",
                        "dest_vrf",
                    ]:
                        exit_point[x.replace("-", "_")] = self.parse_attrib(
                            route_entry,
                            x.replace("dest_vrf", "vrf"),
                        )

                    route["next_hops"].append(exit_point)

                routes.append(route)
                address_family["routes"] = sorted(
                    routes,
                    key=lambda i: i["dest"],
                )
            config["address_families"].append(address_family)

        return utils.remove_empties(config)

    def parse_af(self, item):
        match = re.search(r"(?:\s*)(\w+)(?:\s*)(\w+)", item, re.M)
        if match:
            return match.group(1), match.group(2)

    def parse_faddr(self, item):
        for x in item.split(" "):
            if (":" in x or "." in x) and "/" not in x:
                return x

    def parse_intf(self, item):
        inf_search_strs = [
            r" ((\w+)((?:\d)/(?:\d)/(?:\d)/(?:\d+)))",
            r" (([a-zA-Z]+)(?:\d+))",
        ]
        for i in inf_search_strs:
            match = re.search(i, item, re.M)
            if match:
                return match.group(1)

    def parse_attrib(self, item, attrib):
        match = re.search(r" %s (\S+)" % attrib, item)
        if match:
            val = match.group(1).strip("'")
            if attrib in ["tunnel-id", "vrflabel", "tag", "metric"]:
                val = int(val)
            return val

    def parse_admin_distance(self, item):
        split_item = item.split(" ")
        for item in [
            "vrf",
            "metric",
            "tunnel-id",
            "vrflabel",
            "track",
            "tag",
            "description",
        ]:
            try:
                del split_item[split_item.index(item) + 1]
                del split_item[split_item.index(item)]
            except ValueError:
                continue
        try:
            return [
                i
                for i in split_item
                if "." not in i and ":" not in i and ord(i[0]) > 48 and ord(i[0]) < 57
            ][0]
        except IndexError:
            return None



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: endpoint_analytics_profiling_rules_bulk_v1
short_description: Resource module for Endpoint Analytics Profiling Rules Bulk V1
description:
- Manage operation create of the resource Endpoint Analytics Profiling Rules Bulk V1.
- >
   This API imports the given list of profiling rules. For each record, 1- If 'ruleType' for a record is not 'Custom
   Rule', then it is rejected. 2- If 'ruleId' is provided in the input record,.
version_added: '6.16.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  profilingRules:
    description: Endpoint Analytics Profiling Rules Bulk's profilingRules.
    elements: dict
    suboptions:
      clusterId:
        description: Unique identifier for ML cluster. Only applicable for 'ML Rule'.
        type: str
      conditionGroups:
        description: Endpoint Analytics Profiling Rules Bulk's conditionGroups.
        suboptions:
          condition:
            description: Endpoint Analytics Profiling Rules Bulk's condition.
            suboptions:
              attribute:
                description: Endpoint Analytics Profiling Rules Bulk's attribute.
                type: str
              attributeDictionary:
                description: Endpoint Analytics Profiling Rules Bulk's attributeDictionary.
                type: str
              operator:
                description: Endpoint Analytics Profiling Rules Bulk's operator.
                type: str
              value:
                description: Endpoint Analytics Profiling Rules Bulk's value.
                type: str
            type: dict
          conditionGroup:
            description: Endpoint Analytics Profiling Rules Bulk's conditionGroup.
            elements: str
            type: list
          operator:
            description: Endpoint Analytics Profiling Rules Bulk's operator.
            type: str
          type:
            description: Endpoint Analytics Profiling Rules Bulk's type.
            type: str
        type: dict
      isDeleted:
        description: Flag to indicate whether the rule was deleted.
        type: bool
      lastModifiedBy:
        description: User that last modified the rule. It is read-only, and is ignored
          if provided as part of input request.
        type: str
      lastModifiedOn:
        description: Timestamp (in epoch milliseconds) of last modification. It is read-only,
          and is ignored if provided as part of input request.
        type: int
      pluginId:
        description: Plugin for the rule. Only applicable for 'Cisco Default' rules.
        type: str
      rejected:
        description: Flag to indicate whether rule has been rejected by user or not.
          Only applicable for 'ML Rule'.
        type: bool
      result:
        description: Endpoint Analytics Profiling Rules Bulk's result.
        suboptions:
          deviceType:
            description: List of device types determined by the current rule.
            elements: str
            type: list
          hardwareManufacturer:
            description: List of hardware manufacturers determined by the current rule.
            elements: str
            type: list
          hardwareModel:
            description: List of hardware models determined by the current rule.
            elements: str
            type: list
          operatingSystem:
            description: List of operating systems determined by the current rule.
            elements: str
            type: list
        type: dict
      ruleId:
        description: Unique identifier for the rule. This is normally generated by the
          system, and client does not need to provide it for rules that need to be newly
          created.
        type: str
      ruleName:
        description: Human readable name for the rule.
        type: str
      rulePriority:
        description: Priority for the rule.
        type: int
      ruleType:
        description: Type of the rule.
        type: str
      ruleVersion:
        description: Version of the rule.
        type: int
      sourcePriority:
        description: Source priority for the rule.
        type: int
      usedAttributes:
        description: List of attributes used in the rule. Only applicable for 'Cisco
          Default' rules.
        elements: str
        type: list
    type: list
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
notes:
  - SDK Method used are
    ai_endpoint_analytics.AIEndpointAnalytics.import_profiling_rules_in_bulk_v1,

  - Paths used are
    post /dna/intent/api/v1/endpoint-analytics/profiling-rules/bulk,

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.endpoint_analytics_profiling_rules_bulk_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    profilingRules:
    - clusterId: string
      conditionGroups:
        condition:
          attribute: string
          attributeDictionary: string
          operator: string
          value: string
        conditionGroup:
        - string
        operator: string
        type: string
      isDeleted: true
      lastModifiedBy: string
      lastModifiedOn: 0
      pluginId: string
      rejected: true
      result:
        deviceType:
        - string
        hardwareManufacturer:
        - string
        hardwareModel:
        - string
        operatingSystem:
        - string
      ruleId: string
      ruleName: string
      rulePriority: 0
      ruleType: string
      ruleVersion: 0
      sourcePriority: 0
      usedAttributes:
      - string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {}
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: network_devices_info
short_description: Information module for Network Devices Info
description:
- This module represents an alias of the module network_devices_v1_info
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  startTime:
    description:
    - >
      StartTime query parameter. Start time from which API queries the data set related to the resource. It must
      be specified in UNIX epochtime in milliseconds. Value is inclusive. If `startTime` is not provided, API will
      default to current time.
    type: float
  endTime:
    description:
    - >
      EndTime query parameter. End time to which API queries the data set related to the resource. It must be
      specified in UNIX epochtime in milliseconds. Value is inclusive.
    type: float
  limit:
    description:
    - Limit query parameter. Maximum number of records to return.
    type: float
  offset:
    description:
    - >
      Offset query parameter. Specifies the starting point within all records returned by the API. It's one based
      offset. The starting value is 1.
    type: float
  sortBy:
    description:
    - SortBy query parameter. A field within the response to sort by.
    type: str
  order:
    description:
    - Order query parameter. The sort order of the field ascending or descending.
    type: str
  siteHierarchy:
    description:
    - >
      SiteHierarchy query parameter. The full hierarchical breakdown of the site tree starting from Global site
      name and ending with the specific site name. The Root site is named "Global" (Ex.
      `Global/AreaName/BuildingName/FloorName`) This field supports wildcard asterisk (*) character search
      support. E.g. */San*, */San, /San* Examples `?siteHierarchy=Global/AreaName/BuildingName/FloorName` (single
      siteHierarchy requested) `?siteHierarchy=Global/AreaName/BuildingName/FloorName&siteHierarchy=Global/AreaNam
      e2/BuildingName2/FloorName2` (multiple siteHierarchies requested).
    type: str
  siteHierarchyId:
    description:
    - >
      SiteHierarchyId query parameter. The full hierarchy breakdown of the site tree in id form starting from
      Global site UUID and ending with the specific site UUID. (Ex. `globalUuid/areaUuid/buildingUuid/floorUuid`)
      This field supports wildcard asterisk (*) character search support. E.g. `*uuid*, *uuid, uuid* Examples
      `?siteHierarchyId=globalUuid/areaUuid/buildingUuid/floorUuid `(single siteHierarchyId requested) `?siteHiera
      rchyId=globalUuid/areaUuid/buildingUuid/floorUuid&siteHierarchyId=globalUuid/areaUuid2/buildingUuid2/floorUu
      id2` (multiple siteHierarchyIds requested).
    type: str
  siteId:
    description:
    - >
      SiteId query parameter. The UUID of the site. (Ex. `flooruuid`) This field supports wildcard asterisk (*)
      character search support. E.g.*flooruuid*, *flooruuid, flooruuid* Examples `?siteId=id1` (single id
      requested) `?siteId=id1&siteId=id2&siteId=id3` (multiple ids requested).
    type: str
  id:
    description:
    - >
      Id query parameter. The list of entity Uuids. (Ex."6bef213c-19ca-4170-8375-b694e251101c") Examples
      id=6bef213c-19ca-4170-8375-b694e251101c (single entity uuid requested) id=6bef213c-19ca-4170-8375-
      b694e251101c&id=32219612-819e-4b5e-a96b-cf22aca13dd9&id=2541e9a7-b80d-4955-8aa2-79b233318ba0 (multiple
      entity uuid with '&' separator).
    type: str
  managementIpAddress:
    description:
    - >
      ManagementIpAddress query parameter. The list of entity management IP Address. It can be either Ipv4 or Ipv6
      address or combination of both(Ex. "121.1.1.10") This field supports wildcard (`*`) character-based search.
      Ex `*1.1*` or `1.1*` or `*1.1` Examples managementIpAddresses=121.1.1.10
      managementIpAddresses=121.1.1.10&managementIpAddresses=172.20.1.10&managementIpAddresses=200
      10&=managementIpAddresses172.20.3.4 (multiple entity IP Address with & separator).
    type: str
  macAddress:
    description:
    - >
      MacAddress query parameter. The macAddress of the network device or client This field supports wildcard
      (`*`) character-based search. Ex `*AB AB AB*` or `AB AB AB*` or `*AB AB AB` Examples `macAddress=AB AB AB CD
      CD CD` (single macAddress requested) `macAddress=AB AB AB CD CD DC&macAddress=AB AB AB CD CD FE` (multiple
      macAddress requested).
    type: str
  family:
    description:
    - >
      Family query parameter. The list of network device family names Examples family=Switches and Hubs (single
      network device family name )family=Switches and Hubs&family=Router&family=Wireless Controller (multiple
      Network device family names with & separator). This field is not case sensitive.
    type: str
  type:
    description:
    - >
      Type query parameter. The list of network device type This field supports wildcard (`*`) character-based
      search. Ex `*9407R*` or `*9407R` or `9407R*` Examples type=SwitchesCisco Catalyst 9407R Switch (single
      network device types ) type=Cisco Catalyst 38xx stack-able ethernet switch&type=Cisco 3945 Integrated
      Services Router G2 (multiple Network device types with & separator).
    type: str
  role:
    description:
    - >
      Role query parameter. The list of network device role. Examples role=CORE, role=CORE&role=ACCESS&role=ROUTER
      (multiple Network device roles with & separator). This field is not case sensitive.
    type: str
  serialNumber:
    description:
    - >
      SerialNumber query parameter. The list of network device serial numbers. This field supports wildcard (`*`)
      character-based search. Ex `*MS1SV*` or `MS1SV*` or `*MS1SV` Examples serialNumber=9FUFMS1SVAX
      serialNumber=9FUFMS1SVAX&FCW2333Q0BY&FJC240617JX(multiple Network device serial number with & separator).
    type: str
  maintenanceMode:
    description:
    - MaintenanceMode query parameter. The device maintenanceMode status true or false.
    type: bool
  softwareVersion:
    description:
    - >
      SoftwareVersion query parameter. The list of network device software version This field supports wildcard
      (`*`) character-based search. Ex `*17.8*` or `*17.8` or `17.8*` Examples softwareVersion=2.3.4.0 (single
      network device software version ) softwareVersion=17.9.3.23&softwareVersion=17.7.1.2&softwareVersion=*.17.7
      (multiple Network device software versions with & separator).
    type: str
  healthScore:
    description:
    - >
      HealthScore query parameter. The list of entity health score categories Examples healthScore=good,
      healthScore=good&healthScore=fair (multiple entity healthscore values with & separator). This field is not
      case sensitive.
    type: str
  view:
    description:
    - >
      View query parameter. The List of Network Device model views. Please refer to ```NetworkDeviceView``` for
      the supported list.
    type: str
  attribute:
    description:
    - >
      Attribute query parameter. The List of Network Device model attributes. This is helps to specify the
      interested fields in the request.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Devices GetTheDeviceDataForTheGivenDeviceIdUuidV1
  description: Complete reference of the GetTheDeviceDataForTheGivenDeviceIdUuidV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!get-the-device-data-for-the-given-device-id-uuid
- name: Cisco DNA Center documentation for Devices GetsTheNetworkDeviceDetailsBasedOnTheProvidedQueryParametersV1
  description: Complete reference of the GetsTheNetworkDeviceDetailsBasedOnTheProvidedQueryParametersV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!gets-the-network-device-details-based-on-the-provided-query-parameters  # noqa: E501
notes:
  - SDK Method used are
    devices.Devices.get_the_device_data_for_the_given_device_id_uuid_v1,
    devices.Devices.gets_the_network_device_details_based_on_the_provided_query_parameters_v1,

  - Paths used are
    get /dna/data/api/v1/networkDevices,
    get /dna/data/api/v1/networkDevices/{id},
  - It should be noted that this module is an alias of network_devices_v1_info

"""

EXAMPLES = r"""
- name: Get all Network Devices Info
  cisco.dnac.network_devices_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    startTime: 0
    endTime: 0
    limit: 0
    offset: 0
    sortBy: string
    order: string
    siteHierarchy: string
    siteHierarchyId: string
    siteId: string
    id: string
    managementIpAddress: string
    macAddress: string
    family: string
    type: string
    role: string
    serialNumber: string
    maintenanceMode: True
    softwareVersion: string
    healthScore: string
    view: string
    attribute: string
  register: result

- name: Get Network Devices Info by id
  cisco.dnac.network_devices_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    startTime: 0
    endTime: 0
    view: string
    attribute: string
    id: string
  register: result

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "id": "string",
        "name": "string",
        "managementIpAddress": "string",
        "platformId": "string",
        "deviceFamily": "string",
        "serialNumber": "string",
        "macAddress": "string",
        "deviceSeries": "string",
        "softwareVersion": "string",
        "productVendor": "string",
        "deviceRole": "string",
        "deviceType": "string",
        "communicationState": "string",
        "collectionStatus": "string",
        "haStatus": "string",
        "lastBootTime": 0,
        "siteHierarchyId": "string",
        "siteHierarchy": "string",
        "siteId": "string",
        "deviceGroupHierarchyId": "string",
        "tagNames": [
          "string"
        ],
        "stackType": "string",
        "osType": "string",
        "ringStatus": true,
        "maintenanceModeEnabled": true,
        "upTime": 0,
        "ipv4Address": "string",
        "ipv6Address": "string",
        "redundancyMode": "string",
        "featureFlagList": [
          "string"
        ],
        "haLastResetReason": "string",
        "redundancyPeerStateDerived": "string",
        "redundancyPeerState": "string",
        "redundancyStateDerived": "string",
        "redundancyState": "string",
        "wiredClientCount": 0,
        "wirelessClientCount": 0,
        "portCount": 0,
        "clientCount": 0,
        "apDetails": {
          "connectedWlcName": "string",
          "policyTagName": "string",
          "apOperationalState": "string",
          "powerSaveMode": "string",
          "operationalMode": "string",
          "resetReason": "string",
          "protocol": "string",
          "powerMode": "string",
          "connectedTime": 0,
          "ledFlashEnabled": true,
          "ledFlashSeconds": 0,
          "subMode": "string",
          "homeApEnabled": true,
          "powerType": "string",
          "apType": "string",
          "adminState": "string",
          "icapCapability": "string",
          "regulatoryDomain": "string",
          "ethernetMac": "string",
          "rfTagName": "string",
          "siteTagName": "string",
          "powerSaveModeCapable": "string",
          "powerProfile": "string",
          "flexGroup": "string",
          "powerCalendarProfile": "string",
          "apGroup": "string",
          "radios": [
            {
              "id": "string",
              "band": "string",
              "noise": 0,
              "airQuality": 0,
              "interference": 0,
              "trafficUtil": 0,
              "utilization": 0,
              "clientCount": 0
            }
          ]
        },
        "metricsDetails": {
          "overallHealthScore": 0,
          "overallFabricScore": 0,
          "cpuUtilization": 0,
          "cpuScore": 0,
          "memoryUtilization": 0,
          "memoryScore": 0,
          "avgTemperature": 0,
          "maxTemperature": 0,
          "discardScore": 0,
          "discardInterfaces": [
            "string"
          ],
          "errorScore": 0,
          "errorInterfaces": [
            "string"
          ],
          "interDeviceLinkScore": 0,
          "interDeviceConnectedDownInterfaces": [
            "string"
          ],
          "linkUtilizationScore": 0,
          "highLinkUtilizationInterfaces": [
            "string"
          ],
          "freeTimerScore": 0,
          "freeTimer": 0,
          "packetPoolScore": 0,
          "packetPool": 0,
          "freeMemoryBufferScore": 0,
          "freeMemoryBuffer": 0,
          "wqePoolScore": 0,
          "wqePool": 0,
          "apCount": 0,
          "noiseScore": 0,
          "utilizationScore": 0,
          "interferenceScore": 0,
          "airQualityScore": 0
        },
        "fabricDetails": {
          "fabricRole": [
            "string"
          ],
          "fabricSiteName": "string",
          "transitFabrics": [
            "string"
          ]
        },
        "aggregateAttributes": [
          {
            "name": "string",
            "function": "string",
            "value": 0
          }
        ]
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: user_enrichment_details_info
short_description: Information module for User Enrichment Details Info
description:
- This module represents an alias of the module user_enrichment_details_v1_info
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Users GetUserEnrichmentDetailsV1
  description: Complete reference of the GetUserEnrichmentDetailsV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!get-user-enrichment-details
notes:
  - SDK Method used are
    users.Users.get_user_enrichment_details_v1,

  - Paths used are
    get /dna/intent/api/v1/user-enrichment-details,
  - It should be noted that this module is an alias of user_enrichment_details_v1_info

"""

EXAMPLES = r"""
- name: Get all User Enrichment Details Info
  cisco.dnac.user_enrichment_details_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
  register: result

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: list
  elements: dict
  sample: >
    [
      {
        "userDetails": {
          "id": "string",
          "connectionStatus": "string",
          "hostType": "string",
          "userId": {},
          "hostName": {},
          "hostOs": {},
          "hostVersion": {},
          "subType": "string",
          "lastUpdated": 0,
          "healthScore": [
            {
              "healthType": "string",
              "reason": "string",
              "score": 0
            }
          ],
          "hostMac": "string",
          "hostIpV4": "string",
          "hostIpV6": [
            {}
          ],
          "authType": {},
          "vlanId": "string",
          "ssid": {},
          "frequency": {},
          "channel": {},
          "apGroup": {},
          "location": {},
          "clientConnection": "string",
          "connectedDevice": [
            {}
          ],
          "issueCount": 0,
          "rssi": {},
          "avgRssi": {},
          "snr": {},
          "avgSnr": {},
          "dataRate": {},
          "txBytes": {},
          "rxBytes": {},
          "dnsSuccess": {},
          "dnsFailure": {},
          "onboarding": {
            "averageRunDuration": {},
            "maxRunDuration": {},
            "averageAssocDuration": {},
            "maxAssocDuration": {},
            "averageAuthDuration": {},
            "maxAuthDuration": {},
            "averageDhcpDuration": {},
            "maxDhcpDuration": {},
            "aaaServerIp": {},
            "dhcpServerIp": {}
          },
          "onboardingTime": {},
          "port": {}
        },
        "connectedDevice": [
          {
            "deviceDetails": {
              "family": "string",
              "type": "string",
              "location": {},
              "errorCode": {},
              "macAddress": "string",
              "role": "string",
              "apManagerInterfaceIp": "string",
              "associatedWlcIp": "string",
              "bootDateTime": "string",
              "collectionStatus": "string",
              "interfaceCount": "string",
              "lineCardCount": "string",
              "lineCardId": "string",
              "managementIpAddress": "string",
              "memorySize": "string",
              "platformId": "string",
              "reachabilityFailureReason": "string",
              "reachabilityStatus": "string",
              "snmpContact": "string",
              "snmpLocation": "string",
              "tunnelUdpPort": {},
              "waasDeviceMode": {},
              "series": "string",
              "inventoryStatusDetail": "string",
              "collectionInterval": "string",
              "serialNumber": "string",
              "softwareVersion": "string",
              "roleSource": "string",
              "hostname": "string",
              "upTime": "string",
              "lastUpdateTime": 0,
              "errorDescription": {},
              "locationName": {},
              "tagCount": "string",
              "lastUpdated": "string",
              "instanceUuid": "string",
              "id": "string",
              "neighborTopology": [
                {
                  "errorCode": 0,
                  "message": "string",
                  "detail": "string"
                }
              ]
            }
          }
        ]
      }
    ]
"""



# -*- coding: utf-8 -*-
# Copyright 2024 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

"""
The Vrf_global parser templates file. This contains
a list of parser definitions and associated functions that
facilitates both facts gathering and native command generation for
the given network resource.
"""

import re

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.network_template import (
    NetworkTemplate,
)


class Vrf_globalTemplate(NetworkTemplate):
    def __init__(self, lines=None, module=None):
        super(Vrf_globalTemplate, self).__init__(
            lines=lines,
            tmplt=self,
            module=module,
        )

    # fmt: off
    PARSERS = [
        {
            "name": "name",
            "getval": re.compile(
                r"""
                ^vrf\sdefinition
                (\s(?P<name>\S+))?
                $""", re.VERBOSE,
            ),
            "setval": "vrf definition {{ name }}",
            "result": {
                "vrfs": {
                    '{{ name }}': {
                        'name': '{{ name }}',
                    },
                },
            },
            "shared": True,
        },
        {
            "name": "description",
            "getval": re.compile(
                r"""
                \s+description\s(?P<description>.+$)
                $""", re.VERBOSE,
            ),
            "setval": "description {{ description }}",
            "result": {
                "vrfs": {
                    '{{ name }}': {
                        'name': '{{ name }}',
                        'description': '{{ description }}',
                    },
                },
            },
        },
        {
            "name": "ipv4.multicast.multitopology",
            "getval": re.compile(
                r"""
                \s+ipv4\smulticast\s(?P<multitopology>multitopology)
                $""", re.VERBOSE,
            ),
            "setval": "ipv4 multicast multitopology",
            "result": {
                "vrfs": {
                    '{{ name }}': {
                        'name': '{{ name }}',
                        'ipv4': {
                            'multicast': {
                                'multitopology': "{{ true if multitopology is defined }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "ipv6.multicast.multitopology",
            "getval": re.compile(
                r"""
                \s+ipv6\smulticast\s(?P<multitopology>multitopology)
                $""", re.VERBOSE,
            ),
            "setval": "ipv6 multicast multitopology",
            "result": {
                "vrfs": {
                    '{{ name }}': {
                        'name': '{{ name }}',
                        'ipv6': {
                            'multicast': {
                                'multitopology': "{{ true if multitopology is defined }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "rd",
            "getval": re.compile(
                r"""
                \s+rd\s(?P<rd>\S+)
                $""", re.VERBOSE,
            ),
            "setval": "rd {{ rd }}",
            "result": {
                "vrfs": {
                    '{{ name }}': {
                        'name': '{{ name }}',
                        "rd": "{{ rd }}",
                    },
                },
            },
        },
        {
            "name": "route_target.export",
            "getval": re.compile(
                r"""
                \s+route-target\sexport\s(?P<route_target_export>\S+)
                $""", re.VERBOSE,
            ),
            "setval": "route-target export {{ route_target.export }}",
            "result": {
                "vrfs": {
                    '{{ name }}': {
                        'name': '{{ name }}',
                        "route_target": {
                            "export": "{{ route_target_export }}",
                        },
                    },
                },
            },
        },
        {
            "name": "route_target.import_config",
            "getval": re.compile(
                r"""
                \s+route-target\simport\s(?P<route_target_import_config>\S+)
                $""", re.VERBOSE,
            ),
            "setval": "route-target import {{ route_target.import_config}}",
            "result": {
                "vrfs": {
                    '{{ name }}': {
                        'name': '{{ name }}',
                        "route_target": {
                            "import_config": "{{ route_target_import_config }}",
                        },
                    },
                },
            },
        },
        {
            "name": "route_target.both",
            "getval": re.compile(
                r"""
                \s+route-target\sboth\s(?P<route_target_both>\S+)
                $""", re.VERBOSE,
            ),
            "setval": "route-target both {{ route_target.both }}",
            "result": {
                "vrfs": {
                    '{{ name }}': {
                        'name': '{{ name }}',
                        "route_target": {
                            "both": "{{ route_target_both }}",
                        },
                    },
                },
            },
        },
        {
            "name": "vnet.tag",
            "getval": re.compile(
                r"""
                \s+vnet\stag\s(?P<vnet_tag>\d+)
                $""", re.VERBOSE,
            ),
            "setval": "vnet tag {{ vnet.tag }}",
            "result": {
                "vrfs": {
                    '{{ name }}': {
                        'name': '{{ name }}',
                        "vnet": {
                            "tag": "{{ vnet_tag }}",
                        },
                    },
                },
            },
        },
        {
            "name": "vpn.id",
            "getval": re.compile(
                r"""
                \s+vpn\sid\s(?P<vpn_id>\S+)
                $""", re.VERBOSE,
            ),
            "setval": "vpn id {{ vpn.id }}",
            "result": {
                "vrfs": {
                    '{{ name }}': {
                        'name': '{{ name }}',
                        "vpn": {
                            "id": "{{ vpn_id }}",
                        },
                    },
                },
            },
        },
    ]
    # fmt: on



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: event_subscription_info
short_description: Information module for Event Subscription Info
description:
- This module represents an alias of the module event_subscription_v1_info
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  eventIds:
    description:
    - EventIds query parameter. List of subscriptions related to the respective eventIds.
    type: str
  offset:
    description:
    - Offset query parameter. The number of Subscriptions's to offset in the resultset whose default value 0.
    type: float
  limit:
    description:
    - Limit query parameter. The number of Subscriptions's to limit in the resultset whose default value 10.
    type: float
  sortBy:
    description:
    - SortBy query parameter. SortBy field name.
    type: str
  order:
    description:
    - Order query parameter.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Event Management GetEventSubscriptionsV1
  description: Complete reference of the GetEventSubscriptionsV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!get-event-subscriptions
notes:
  - SDK Method used are
    event_management.EventManagement.get_event_subscriptions_v1,

  - Paths used are
    get /dna/intent/api/v1/event/subscription,
  - It should be noted that this module is an alias of event_subscription_v1_info

"""

EXAMPLES = r"""
- name: Get all Event Subscription Info
  cisco.dnac.event_subscription_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    eventIds: string
    offset: 0
    limit: 0
    sortBy: string
    order: string
  register: result

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: list
  elements: dict
  sample: >
    [
      {
        "version": "string",
        "subscriptionId": "string",
        "name": "string",
        "description": "string",
        "subscriptionEndpoints": [
          {
            "instanceId": "string",
            "subscriptionDetails": {
              "connectorType": "string",
              "instanceId": "string",
              "name": "string",
              "description": "string",
              "url": "string",
              "basePath": "string",
              "resource": "string",
              "method": "string",
              "trustCert": true,
              "headers": [
                {
                  "string": "string"
                }
              ],
              "queryParams": [
                {
                  "string": "string"
                }
              ],
              "pathParams": [
                {
                  "string": "string"
                }
              ],
              "body": "string",
              "connectTimeout": 0,
              "readTimeout": 0
            },
            "connectorType": "string"
          }
        ],
        "filter": {
          "eventIds": [
            "string"
          ],
          "others": [
            "string"
          ],
          "domainsSubdomains": [
            {
              "domain": "string",
              "subDomains": [
                "string"
              ]
            }
          ],
          "types": [
            "string"
          ],
          "categories": [
            "string"
          ],
          "severities": [
            "string"
          ],
          "sources": [
            "string"
          ],
          "siteIds": [
            "string"
          ]
        },
        "isPrivate": true,
        "tenantId": "string"
      }
    ]
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: site_health_summaries_info
short_description: Information module for Site Health Summaries Info
description:
- This module represents an alias of the module site_health_summaries_v1_info
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  startTime:
    description:
    - >
      StartTime query parameter. Start time from which API queries the data set related to the resource. It must
      be specified in UNIX epochtime in milliseconds. Value is inclusive. If `startTime` is not provided, API will
      default to current time.
    type: float
  endTime:
    description:
    - >
      EndTime query parameter. End time to which API queries the data set related to the resource. It must be
      specified in UNIX epochtime in milliseconds. Value is inclusive.
    type: float
  limit:
    description:
    - Limit query parameter. Maximum number of records to return.
    type: float
  offset:
    description:
    - >
      Offset query parameter. Specifies the starting point within all records returned by the API. It's one based
      offset. The starting value is 1.
    type: float
  sortBy:
    description:
    - SortBy query parameter. A field within the response to sort by.
    type: str
  order:
    description:
    - Order query parameter. The sort order of the field ascending or descending.
    type: str
  siteHierarchy:
    description:
    - >
      SiteHierarchy query parameter. The full hierarchical breakdown of the site tree starting from Global site
      name and ending with the specific site name. The Root site is named "Global" (Ex.
      `Global/AreaName/BuildingName/FloorName`) This field supports wildcard asterisk (`*`) character search
      support. E.g. `*/San*, */San, /San*` Examples `?siteHierarchy=Global/AreaName/BuildingName/FloorName`
      (single siteHierarchy requested) `?siteHierarchy=Global/AreaName/BuildingName/FloorName&siteHierarchy=Global
      /AreaName2/BuildingName2/FloorName2` (multiple siteHierarchies requested).
    type: str
  siteHierarchyId:
    description:
    - >
      SiteHierarchyId query parameter. The full hierarchy breakdown of the site tree in id form starting from
      Global site UUID and ending with the specific site UUID. (Ex. `globalUuid/areaUuid/buildingUuid/floorUuid`)
      This field supports wildcard asterisk (`*`) character search support. E.g. `*uuid*, *uuid, uuid*` Examples
      `?siteHierarchyId=globalUuid/areaUuid/buildingUuid/floorUuid `(single siteHierarchyId requested) `?siteHiera
      rchyId=globalUuid/areaUuid/buildingUuid/floorUuid&siteHierarchyId=globalUuid/areaUuid2/buildingUuid2/floorUu
      id2` (multiple siteHierarchyIds requested).
    type: str
  siteType:
    description:
    - >
      SiteType query parameter. The type of the site. A site can be an area, building, or floor. Default when not
      provided will be `floor,building,area` Examples `?siteType=area` (single siteType requested)
      `?siteType=area&siteType=building&siteType=floor` (multiple siteTypes requested).
    type: str
  id:
    description:
    - >
      Id query parameter. The list of entity Uuids. (Ex."6bef213c-19ca-4170-8375-b694e251101c") Examples
      id=6bef213c-19ca-4170-8375-b694e251101c (single entity uuid requested) id=6bef213c-19ca-4170-8375-
      b694e251101c&id=32219612-819e-4b5e-a96b-cf22aca13dd9&id=2541e9a7-b80d-4955-8aa2-79b233318ba0 (multiple
      entity uuid with '&' separator).
    type: str
  view:
    description:
    - >
      View query parameter. The specific summary view being requested. This is an optional parameter which can be
      passed to get one or more of the specific health data summaries associated with sites. ### Response data
      proviced by each view 1. **site** id, siteHierarchy, siteHierarchyId, siteType, latitude, longitude 2.
      **network** id, networkDeviceCount, networkDeviceGoodHealthCount,wirelessDeviceCount,
      wirelessDeviceGoodHealthCount, accessDeviceCount, accessDeviceGoodHealthCount, coreDeviceCount,
      coreDeviceGoodHealthCount, distributionDeviceCount, distributionDeviceGoodHealthCount, routerDeviceCount,
      routerDeviceGoodHealthCount, apDeviceCount, apDeviceGoodHealthCount, wlcDeviceCount,
      wlcDeviceGoodHealthCount, switchDeviceCount, switchDeviceGoodHealthCount, networkDeviceGoodHealthPercentage,
      accessDeviceGoodHealthPercentage, coreDeviceGoodHealthPercentage, distributionDeviceGoodHealthPercentage,
      routerDeviceGoodHealthPercentage, apDeviceGoodHealthPercentage, wlcDeviceGoodHealthPercentage,
      switchDeviceGoodHealthPercentage, wirelessDeviceGoodHealthPercentage 3. **client** id, clientCount,
      clientGoodHealthCount, wiredClientCount, wirelessClientCount, wiredClientGoodHealthCount,
      wirelessClientGoodHealthCount, clientGoodHealthPercentage, wiredClientGoodHealthPercentage,
      wirelessClientGoodHealthPercentage, clientDataUsage 4. **issue** id, p1IssueCount, p2IssueCount,
      p3IssueCount, p4IssueCount, issueCount When this query parameter is not added the default summaries are
      **site,client,network,issue** Examples view=client (single view requested)
      view=client&view=network&view=issue (multiple views requested).
    type: str
  attribute:
    description:
    - >
      Attribute query parameter. Supported Attributes id, siteHierarchy, siteHierarchyId, siteType, latitude,
      longitude, networkDeviceCount, networkDeviceGoodHealthCount,wirelessDeviceCount,
      wirelessDeviceGoodHealthCount, accessDeviceCount, accessDeviceGoodHealthCount, coreDeviceCount,
      coreDeviceGoodHealthCount, distributionDeviceCount, distributionDeviceGoodHealthCount, routerDeviceCount,
      routerDeviceGoodHealthCount, apDeviceCount, apDeviceGoodHealthCount, wlcDeviceCount,
      wlcDeviceGoodHealthCount, switchDeviceCount, switchDeviceGoodHealthCount, networkDeviceGoodHealthPercentage,
      accessDeviceGoodHealthPercentage, coreDeviceGoodHealthPercentage, distributionDeviceGoodHealthPercentage,
      routerDeviceGoodHealthPercentage, apDeviceGoodHealthPercentage, wlcDeviceGoodHealthPercentage,
      switchDeviceGoodHealthPercentage, wirelessDeviceGoodHealthPercentage, clientCount, clientGoodHealthCount,
      wiredClientCount, wirelessClientCount, wiredClientGoodHealthCount, wirelessClientGoodHealthCount,
      clientGoodHealthPercentage, wiredClientGoodHealthPercentage, wirelessClientGoodHealthPercentage,
      clientDataUsage, p1IssueCount, p2IssueCount, p3IssueCount, p4IssueCount, issueCount If length of attribute
      list is too long, please use 'view' param instead. Examples attribute=siteHierarchy (single attribute
      requested) attribute=siteHierarchy&attribute=clientCount (multiple attributes requested).
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Sites ReadListOfSiteHealthSummariesV1
  description: Complete reference of the ReadListOfSiteHealthSummariesV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!read-list-of-site-health-summaries
- name: Cisco DNA Center documentation for Sites ReadSiteHealthSummaryDataBySiteIdV1
  description: Complete reference of the ReadSiteHealthSummaryDataBySiteIdV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!read-site-health-summary-data-by-site-id
notes:
  - SDK Method used are
    sites.Sites.read_list_of_site_health_summaries_v1,
    sites.Sites.read_site_health_summary_data_by_site_id_v1,

  - Paths used are
    get /dna/data/api/v1/siteHealthSummaries,
    get /dna/data/api/v1/siteHealthSummaries/{id},
  - It should be noted that this module is an alias of site_health_summaries_v1_info

"""

EXAMPLES = r"""
- name: Get all Site Health Summaries Info
  cisco.dnac.site_health_summaries_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    startTime: 0
    endTime: 0
    limit: 0
    offset: 0
    sortBy: string
    order: string
    siteHierarchy: string
    siteHierarchyId: string
    siteType: string
    id: string
    view: string
    attribute: string
  register: result

- name: Get Site Health Summaries Info by id
  cisco.dnac.site_health_summaries_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    startTime: 0
    endTime: 0
    view: string
    attribute: string
    id: string
  register: result

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "id": "string",
        "siteHierarchy": "string",
        "siteHierarchyId": "string",
        "siteType": "string",
        "latitude": 0,
        "longitude": 0,
        "networkDeviceGoodHealthPercentage": 0,
        "networkDeviceGoodHealthCount": 0,
        "clientGoodHealthCount": 0,
        "clientGoodHealthPercentage": 0,
        "wiredClientGoodHealthPercentage": 0,
        "wirelessClientGoodHealthPercentage": 0,
        "clientCount": 0,
        "wiredClientCount": 0,
        "wirelessClientCount": 0,
        "wiredClientGoodHealthCount": 0,
        "wirelessClientGoodHealthCount": 0,
        "networkDeviceCount": 0,
        "accessDeviceCount": 0,
        "accessDeviceGoodHealthCount": 0,
        "coreDeviceCount": 0,
        "coreDeviceGoodHealthCount": 0,
        "distributionDeviceCount": 0,
        "distributionDeviceGoodHealthCount": 0,
        "routerDeviceCount": 0,
        "routerDeviceGoodHealthCount": 0,
        "wirelessDeviceCount": 0,
        "wirelessDeviceGoodHealthCount": 0,
        "apDeviceCount": 0,
        "apDeviceGoodHealthCount": 0,
        "wlcDeviceCount": 0,
        "wlcDeviceGoodHealthCount": 0,
        "switchDeviceCount": 0,
        "switchDeviceGoodHealthCount": 0,
        "accessDeviceGoodHealthPercentage": 0,
        "coreDeviceGoodHealthPercentage": 0,
        "distributionDeviceGoodHealthPercentage": 0,
        "routerDeviceGoodHealthPercentage": 0,
        "apDeviceGoodHealthPercentage": 0,
        "wlcDeviceGoodHealthPercentage": 0,
        "switchDeviceGoodHealthPercentage": 0,
        "wirelessDeviceGoodHealthPercentage": 0,
        "clientDataUsage": 0,
        "p1IssueCount": 0,
        "p2IssueCount": 0,
        "p3IssueCount": 0,
        "p4IssueCount": 0,
        "issueCount": 0
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: device_details_v1_info
short_description: Information module for Device Details V1
description:
- Get all Device Details V1.
- Returns detailed Network Device information retrieved by Mac Address, Device Name or UUID for any given point of time.
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  timestamp:
    description:
    - Timestamp query parameter. UTC timestamp of device data in milliseconds.
    type: float
  identifier:
    description:
    - Identifier query parameter. One of "macAddress", "nwDeviceName", "uuid" (case insensitive).
    type: str
  searchBy:
    description:
    - SearchBy query parameter. MAC Address, device name, or UUID of the network device.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Devices GetDeviceDetailV1
  description: Complete reference of the GetDeviceDetailV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!get-device-detail
notes:
  - SDK Method used are
    devices.Devices.get_device_detail_v1,

  - Paths used are
    get /dna/intent/api/v1/device-detail,

"""

EXAMPLES = r"""
- name: Get all Device Details V1
  cisco.dnac.device_details_v1_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    timestamp: 0
    identifier: string
    searchBy: string
  register: result

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "noiseScore": 0,
      "policyTagName": "string",
      "interferenceScore": 0,
      "opState": "string",
      "powerSaveMode": "string",
      "mode": "string",
      "resetReason": "string",
      "nwDeviceRole": "string",
      "protocol": "string",
      "powerMode": "string",
      "connectedTime": "string",
      "ringStatus": true,
      "ledFlashSeconds": "string",
      "ip_addr_managementIpAddr": "string",
      "stackType": "string",
      "subMode": "string",
      "serialNumber": "string",
      "nwDeviceName": "string",
      "deviceGroupHierarchyId": "string",
      "cpu": 0,
      "utilization": "string",
      "nwDeviceId": "string",
      "siteHierarchyGraphId": "string",
      "nwDeviceFamily": "string",
      "macAddress": "string",
      "homeApEnabled": "string",
      "deviceSeries": "string",
      "collectionStatus": "string",
      "utilizationScore": 0,
      "maintenanceMode": true,
      "interference": "string",
      "softwareVersion": "string",
      "tagIdList": [
        {}
      ],
      "powerType": "string",
      "overallHealth": 0,
      "managementIpAddr": "string",
      "memory": "string",
      "communicationState": "string",
      "apType": "string",
      "adminState": "string",
      "noise": "string",
      "icapCapability": "string",
      "regulatoryDomain": "string",
      "ethernetMac": "string",
      "nwDeviceType": "string",
      "airQuality": "string",
      "rfTagName": "string",
      "siteTagName": "string",
      "platformId": "string",
      "upTime": "string",
      "memoryScore": 0,
      "powerSaveModeCapable": "string",
      "powerProfile": "string",
      "airQualityScore": 0,
      "location": "string",
      "flexGroup": "string",
      "lastBootTime": 0,
      "powerCalendarProfile": "string",
      "connectivityStatus": 0,
      "ledFlashEnabled": "string",
      "cpuScore": 0,
      "avgTemperature": 0,
      "maxTemperature": 0,
      "haStatus": "string",
      "osType": "string",
      "timestamp": 0,
      "apGroup": "string",
      "redundancyMode": "string",
      "featureFlagList": [
        "string"
      ],
      "freeMbufScore": 0,
      "HALastResetReason": "string",
      "wqeScore": 0,
      "redundancyPeerStateDerived": "string",
      "freeTimerScore": 0,
      "redundancyPeerState": "string",
      "redundancyStateDerived": "string",
      "redundancyState": "string",
      "packetPoolScore": 0,
      "freeTimer": 0,
      "packetPool": 0,
      "wqe": 0,
      "freeMbuf": 0
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: license_device_license_summary_v1_info
short_description: Information module for License Device License Summary V1
description:
- Get all License Device License Summary V1.
- Show license summary of devices.
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  page_number:
    description:
    - Page_number query parameter. Page number of response.
    type: float
  order:
    description:
    - Order query parameter. Sorting order.
    type: str
  sort_by:
    description:
    - Sort_by query parameter. Sort result by field.
    type: str
  dna_level:
    description:
    - Dna_level query parameter. Device Cisco DNA license level. The valid values are Advantage, Essentials.
    type: str
  device_type:
    description:
    - Device_type query parameter. Type of device. The valid values are Routers, Switches and Hubs, Wireless Controller. # noqa: E501
    type: str
  limit:
    description:
    - Limit query parameter.
    type: float
  registration_status:
    description:
    - >
      Registration_status query parameter. Smart license registration status of device. The valid values are
      Unknown, NA, Unregistered, Registered, Registration_expired, Reservation_in_progress, Registered_slr,
      Registered_plr, Registered_satellite.
    type: str
  virtual_account_name:
    description:
    - Virtual_account_name query parameter. Name of virtual account.
    type: str
  smart_account_id:
    description:
    - Smart_account_id query parameter. Id of smart account.
    type: str
  device_uuid:
    description:
    - Device_uuid query parameter. Id of device.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Licenses DeviceLicenseSummaryV1
  description: Complete reference of the DeviceLicenseSummaryV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!device-license-summary
notes:
  - SDK Method used are
    licenses.Licenses.device_license_summary_v1,

  - Paths used are
    get /dna/intent/api/v1/licenses/device/summary,

"""

EXAMPLES = r"""
- name: Get all License Device License Summary V1
  cisco.dnac.license_device_license_summary_v1_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    page_number: 0
    order: string
    sort_by: string
    dna_level: string
    device_type: string
    limit: 0
    registration_status: string
    virtual_account_name: string
    smart_account_id: string
    device_uuid: string
  register: result

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": [
        {
          "authorization_status": "string",
          "last_updated_time": "string",
          "is_performance_allowed": true,
          "sle_auth_code": "string",
          "throughput_level": "string",
          "hsec_status": "string",
          "device_uuid": "string",
          "site": "string",
          "total_access_point_count": 0,
          "model": "string",
          "is_wireless_capable": true,
          "registration_status": "string",
          "sle_state": "string",
          "performance_license": "string",
          "license_mode": "string",
          "is_license_expired": true,
          "software_version": "string",
          "reservation_status": "string",
          "is_wireless": true,
          "network_license": "string",
          "evaluation_license_expiry": "string",
          "wireless_capable_network_license": "string",
          "device_name": "string",
          "device_type": "string",
          "dna_level": "string",
          "virtual_account_name": "string",
          "last_successful_rum_usage_upload_time": "string",
          "ip_address": "string",
          "wireless_capable_dna_license": "string",
          "mac_address": "string",
          "customer_tag1": "string",
          "customer_tag2": "string",
          "customer_tag3": "string",
          "customer_tag4": "string",
          "smart_account_name": "string"
        }
      ],
      "version": "string"
    }
"""



# -*- coding: utf-8 -*-
# Copyright 2019 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
"""
The eos_l3_interfaces class
It is in this file where the current configuration (as dict)
is compared to the provided configuration (as dict) and the command set
necessary to bring the current configuration to it's desired end-state is
created
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.cfg.base import (
    ConfigBase,
)
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.utils import (
    param_list_to_dict,
    to_list,
)

from ansible_collections.arista.eos.plugins.module_utils.network.eos.facts.facts import Facts
from ansible_collections.arista.eos.plugins.module_utils.network.eos.utils.utils import (
    normalize_interface,
)


class L3_interfaces(ConfigBase):
    """
    The eos_l3_interfaces class
    """

    gather_subset = ["!all", "!min"]

    gather_network_resources = ["l3_interfaces"]

    def get_l3_interfaces_facts(self, data=None):
        """Get the 'facts' (the current configuration)

        :rtype: A dictionary
        :returns: The current configuration as a dictionary
        """
        facts, _warnings = Facts(self._module).get_facts(
            self.gather_subset,
            self.gather_network_resources,
            data=data,
        )
        l3_interfaces_facts = facts["ansible_network_resources"].get(
            "l3_interfaces",
        )
        if not l3_interfaces_facts:
            return []
        return l3_interfaces_facts

    def execute_module(self):
        """Execute the module

        :rtype: A dictionary
        :returns: The result from module execution
        """
        result = {"changed": False}
        commands = list()
        warnings = list()

        if self.state in self.ACTION_STATES:
            existing_l3_interfaces_facts = self.get_l3_interfaces_facts()
        else:
            existing_l3_interfaces_facts = []

        if self.state in self.ACTION_STATES or self.state == "rendered":
            commands.extend(self.set_config(existing_l3_interfaces_facts))

        if commands and self.state in self.ACTION_STATES:
            if not self._module.check_mode:
                self._connection.edit_config(commands)
            result["changed"] = True
        if self.state in self.ACTION_STATES:
            result["commands"] = commands

        if self.state in self.ACTION_STATES or self.state == "gathered":
            changed_l3_interfaces_facts = self.get_l3_interfaces_facts()

        elif self.state == "rendered":
            result["rendered"] = commands

        elif self.state == "parsed":
            running_config = self._module.params["running_config"]
            if not running_config:
                self._module.fail_json(
                    msg="value of running_config parameter must not be empty for state parsed",
                )
            result["parsed"] = self.get_l3_interfaces_facts(
                data=running_config,
            )

        if self.state in self.ACTION_STATES:
            result["before"] = existing_l3_interfaces_facts
            if result["changed"]:
                result["after"] = changed_l3_interfaces_facts

        elif self.state == "gathered":
            result["gathered"] = changed_l3_interfaces_facts

        result["warnings"] = warnings
        return result

    def set_config(self, existing_l3_interfaces_facts):
        """Collect the configuration from the args passed to the module,
            collect the current configuration (as a dict from facts)

        :rtype: A list
        :returns: the commands necessary to migrate the current configuration
                  to the desired configuration
        """
        want = self._module.params["config"]
        have = existing_l3_interfaces_facts
        resp = self.set_state(want, have)
        return to_list(resp)

    def set_state(self, want, have):
        """Select the appropriate function based on the state provided

        :param want: the desired configuration as a dictionary
        :param have: the current configuration as a dictionary
        :rtype: A list
        :returns: the commands necessary to migrate the current configuration
                  to the desired configuration
        """
        state = self._module.params["state"]
        if state in ("merged", "replaced", "overridden", "rendered") and not want:
            self._module.fail_json(
                msg="value of config parameter must not be empty for state {0}".format(
                    state,
                ),
            )
        want = param_list_to_dict(want)
        have = param_list_to_dict(have)
        commands = []
        if state == "overridden":
            commands = self._state_overridden(want, have)
        elif state == "deleted":
            commands = self._state_deleted(want, have)
        elif state == "merged" or state == "rendered":
            commands = self._state_merged(want, have)
        elif state == "replaced":
            commands = self._state_replaced(want, have)
        return commands

    @staticmethod
    def _state_replaced(want, have):
        """The command generator when state is replaced

        :rtype: A list
        :returns: the commands necessary to migrate the current configuration
                  to the desired configuration
        """
        commands = []
        for key, desired in want.items():
            interface_name = normalize_interface(key)
            if interface_name in have:
                extant = have[interface_name]
            else:
                extant = dict()
            intf_commands = set_interface(desired, extant)
            intf_commands.extend(clear_interface(desired, extant))

            if intf_commands:
                commands.append("interface {0}".format(interface_name))
                commands.extend(intf_commands)
        return commands

    @staticmethod
    def _state_overridden(want, have):
        """The command generator when state is overridden

        :rtype: A list
        :returns: the commands necessary to migrate the current configuration
                  to the desired configuration
        """
        commands = []
        for key, extant in have.items():
            interface_name = normalize_interface(key)
            desired = dict()
            if "Management" in interface_name:
                continue
            for k in want.keys():
                k_want = normalize_interface(k)
                if key == k_want:
                    desired = want[k]
                    break
            if desired.get("ipv4"):
                for ipv4 in desired["ipv4"]:
                    for k in ["secondary", "virtual"]:
                        if ipv4[k] is None:
                            del ipv4[k]
            intf_commands = set_interface(desired, extant)
            intf_commands.extend(clear_interface(desired, extant))

            if intf_commands:
                commands.append("interface {0}".format(interface_name))
                commands.extend(intf_commands)

        # new interfaces in want
        new_intf_commands = []
        for key, desired in want.items():
            interface_name = normalize_interface(key)
            if interface_name not in have:
                extant = dict()
                new_intf_commands = set_interface(desired, extant)

            if new_intf_commands:
                commands.append("interface {0}".format(interface_name))
                commands.extend(new_intf_commands)

        return commands

    @staticmethod
    def _state_merged(want, have):
        """The command generator when state is merged

        :rtype: A list
        :returns: the commands necessary to merge the provided into
                  the current configuration
        """
        commands = []
        for key, desired in want.items():
            interface_name = normalize_interface(key)
            if interface_name in have:
                extant = have[interface_name]
            else:
                extant = dict()

            intf_commands = set_interface(desired, extant)
            if intf_commands:
                commands.append("interface {0}".format(interface_name))
                commands.extend(intf_commands)

        return commands

    @staticmethod
    def _state_deleted(want, have):
        """The command generator when state is deleted

        :rtype: A list
        :returns: the commands necessary to remove the current configuration
                  of the provided objects
        """
        commands = []
        for key in want:
            interface_name = normalize_interface(key)
            desired = dict()
            if interface_name in have:
                extant = have[interface_name]
            else:
                continue

            intf_commands = clear_interface(desired, extant)

            if intf_commands:
                commands.append("interface {0}".format(interface_name))
                commands.extend(intf_commands)

        return commands


def set_interface(want, have):
    commands = []
    want_ipv4 = set(tuple(sorted(address.items())) for address in want.get("ipv4") or [])
    have_ipv4 = set(tuple(sorted(address.items())) for address in have.get("ipv4") or [])
    for address in want_ipv4 - have_ipv4:
        address = dict(address)
        for param in ["secondary", "virtual"]:
            if param in address and not address[param]:
                del address[param]
        if tuple(sorted(address.items())) in have_ipv4:
            continue

        address_cmd = "ip address {0}".format(address["address"])
        if address.get("secondary"):
            address_cmd += " secondary"
        if address.get("virtual"):
            address_cmd = "ip address virtual {0}".format(address["address"])
        commands.append(address_cmd)

    want_ipv6 = set(tuple(sorted(address.items())) for address in want.get("ipv6") or [])
    have_ipv6 = set(tuple(sorted(address.items())) for address in have.get("ipv6") or [])
    for address in want_ipv6 - have_ipv6:
        address = dict(address)
        commands.append("ipv6 address {0}".format(address["address"]))
    return commands


def clear_interface(want, have):
    commands = []
    want_ipv4 = set(tuple(sorted(address.items())) for address in want.get("ipv4") or [])
    have_ipv4 = set(tuple(sorted(address.items())) for address in have.get("ipv4") or [])
    if not want_ipv4 and have_ipv4:
        commands.append("no ip address")
    else:
        for address in have_ipv4 - want_ipv4:
            address = dict(address)
            for param in ["secondary", "virtual"]:
                if param not in address:
                    address[param] = None
            if tuple(sorted(address.items())) in want_ipv4:
                continue

            if address.get("secondary"):
                commands.append(
                    "no ip address {0} secondary".format(address["address"]),
                )
            if address.get("virtual"):
                commands.append(
                    "no ip address virtual {0}".format(address["address"]),
                )

            if "secondary" not in address:
                # Removing non-secondary removes all other interfaces
                break

    want_ipv6 = set(tuple(sorted(address.items())) for address in want.get("ipv6") or [])
    have_ipv6 = set(tuple(sorted(address.items())) for address in have.get("ipv6") or [])
    for address in have_ipv6 - want_ipv6:
        address = dict(address)
        commands.append("no ipv6 address {0}".format(address["address"]))
    return commands



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: user_info
short_description: Information module for User Info
description:
- This module represents an alias of the module user_v1_info
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  invokeSource:
    description:
    - >
      InvokeSource query parameter. The source that invokes this API. The value of this query parameter must be
      set to "external".
    type: str
  authSource:
    description:
    - >
      AuthSource query parameter. The source that authenticates the user. The value of this query parameter can be
      set to "internal" or "external". If not provided, then all users will be returned in the response.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for User and Roles GetUsersAPIV1
  description: Complete reference of the GetUsersAPIV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!get-users-api
notes:
  - SDK Method used are
    user_and_roles.UserandRoles.get_users_api_v1,

  - Paths used are
    get /dna/system/api/v1/user,
  - It should be noted that this module is an alias of user_v1_info

"""

EXAMPLES = r"""
- name: Get all User Info
  cisco.dnac.user_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    invokeSource: string
    authSource: string
  register: result

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "users": [
        {
          "firstName": "string",
          "lastName": "string",
          "authSource": "string",
          "passphraseUpdateTime": "string",
          "roleList": [
            "string"
          ],
          "userId": "string",
          "email": "string",
          "username": "string"
        }
      ]
    }
"""



# -*- coding: utf-8 -*-

# Copyright: Contributors to the Ansible project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from ..waiter import BaseWaiterFactory

WAITER_MAP = {
    "Standby": "instances_in_standby",
    "Terminated": "instances_terminated",
    "Detached": "instances_detached",
    "InService": "instances_in_service",
    "HEALTHY": "instances_healthy",
    "Healthy": "instances_healthy",
    "UNHEALTHY": "instances_unhealthy",
    "Unhealthy": "instances_unhealthy",
    "Protected": "instances_protected",
    "NotProtected": "instances_not_protected",
}


def _fail_on_instance_lifecycle_states(state):
    return dict(state="failure", matcher="pathAny", expected=state, argument="AutoScalingInstances[].LifecycleState")


def _success_on_instance_lifecycle_states(state):
    return dict(state="success", matcher="pathAll", expected=state, argument="AutoScalingInstances[].LifecycleState")


def _success_on_instance_health(health):
    return dict(state="success", matcher="pathAll", expected=health, argument="AutoScalingInstances[].HealthStatus")


def _success_on_instance_protection(state):
    return dict(
        state="success", matcher="pathAll", expected=state, argument="AutoScalingInstances[].ProtectedFromScaleIn"
    )


def _no_instances(result):
    return dict(state=result, matcher="path", expected=True, argument="length(AutoScalingInstances[]) == `0`")


class AutoscalingWaiterFactory(BaseWaiterFactory):
    @property
    def _waiter_model_data(self):
        data = dict(
            instances_healthy=dict(
                operation="DescribeAutoScalingInstances",
                delay=5,
                maxAttempts=120,
                acceptors=[
                    _success_on_instance_health("HEALTHY"),
                    # Terminated Instances can't reach "Healthy"
                    _fail_on_instance_lifecycle_states("Terminating"),
                    _fail_on_instance_lifecycle_states("Terminated"),
                    _fail_on_instance_lifecycle_states("Terminating:Wait"),
                    _fail_on_instance_lifecycle_states("Terminating:Proceed"),
                ],
            ),
            instances_unhealthy=dict(
                operation="DescribeAutoScalingInstances",
                delay=5,
                maxAttempts=120,
                acceptors=[
                    _success_on_instance_health("UNHEALTHY"),
                    # Instances in an unhealthy state can end up being automatically terminated
                    _no_instances("success"),
                ],
            ),
            instances_protected=dict(
                operation="DescribeAutoScalingInstances",
                delay=5,
                maxAttempts=120,
                acceptors=[
                    _success_on_instance_protection(True),
                ],
            ),
            instances_not_protected=dict(
                operation="DescribeAutoScalingInstances",
                delay=5,
                maxAttempts=120,
                acceptors=[
                    _success_on_instance_protection(False),
                    # Instances without protection can end up being automatically terminated
                    _no_instances("success"),
                ],
            ),
            instances_in_service=dict(
                operation="DescribeAutoScalingInstances",
                delay=5,
                maxAttempts=120,
                acceptors=[
                    _success_on_instance_lifecycle_states("InService"),
                    # Terminated instances can't reach InService
                    _fail_on_instance_lifecycle_states("Terminating"),
                    _fail_on_instance_lifecycle_states("Terminated"),
                    _fail_on_instance_lifecycle_states("Terminating:Wait"),
                    _fail_on_instance_lifecycle_states("Terminating:Proceed"),
                ],
            ),
            instances_in_standby=dict(
                operation="DescribeAutoScalingInstances",
                delay=5,
                maxAttempts=120,
                acceptors=[
                    _success_on_instance_lifecycle_states("Standby"),
                    # Terminated instances can't reach Standby
                    _fail_on_instance_lifecycle_states("Terminating"),
                    _fail_on_instance_lifecycle_states("Terminated"),
                    _fail_on_instance_lifecycle_states("Terminating:Wait"),
                    _fail_on_instance_lifecycle_states("Terminating:Proceed"),
                ],
            ),
            instances_detached=dict(
                operation="DescribeAutoScalingInstances",
                delay=5,
                maxAttempts=120,
                acceptors=[
                    _success_on_instance_lifecycle_states("Detached"),
                    _no_instances("success"),
                ],
            ),
            instances_terminated=dict(
                operation="DescribeAutoScalingInstances",
                delay=5,
                maxAttempts=120,
                acceptors=[
                    _success_on_instance_lifecycle_states("Terminated"),
                    _no_instances("success"),
                ],
            ),
        )

        return data


waiter_factory = AutoscalingWaiterFactory()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: pnp_device_claim
short_description: Resource module for Pnp Device Claim
description:
- This module represents an alias of the module pnp_device_claim_v1
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  authorizationNeeded:
    description: Flag to enable/disable PnP device authorization. (true means enable).
    type: bool
  configFileUrl:
    description: Pnp Device Claim's configFileUrl.
    type: str
  configId:
    description: Pnp Device Claim's configId.
    type: str
  deviceClaimList:
    description: Pnp Device Claim's deviceClaimList.
    elements: dict
    suboptions:
      configList:
        description: Pnp Device Claim's configList.
        elements: dict
        suboptions:
          configId:
            description: Pnp Device Claim's configId.
            type: str
          configParameters:
            description: Pnp Device Claim's configParameters.
            elements: dict
            suboptions:
              key:
                description: Pnp Device Claim's key.
                type: str
              value:
                description: Pnp Device Claim's value.
                type: str
            type: list
        type: list
      deviceId:
        description: Pnp Device Claim's deviceId.
        type: str
      licenseLevel:
        description: Pnp Device Claim's licenseLevel.
        type: str
      licenseType:
        description: Pnp Device Claim's licenseType.
        type: str
      topOfStackSerialNumber:
        description: Pnp Device Claim's topOfStackSerialNumber.
        type: str
    type: list
  fileServiceId:
    description: Pnp Device Claim's fileServiceId.
    type: str
  imageId:
    description: Pnp Device Claim's imageId.
    type: str
  imageUrl:
    description: Pnp Device Claim's imageUrl.
    type: str
  populateInventory:
    description: PopulateInventory flag.
    type: bool
  projectId:
    description: Pnp Device Claim's projectId.
    type: str
  workflowId:
    description: Pnp Device Claim's workflowId.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Device Onboarding (PnP) ClaimDeviceV1
  description: Complete reference of the ClaimDeviceV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!claim-device
notes:
  - SDK Method used are
    device_onboarding_pnp.DeviceOnboardingPnp.claim_device_v1,

  - Paths used are
    post /dna/intent/api/v1/onboarding/pnp-device/claim,
  - It should be noted that this module is an alias of pnp_device_claim_v1

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.pnp_device_claim:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    authorizationNeeded: true
    configFileUrl: string
    configId: string
    deviceClaimList:
    - configList:
      - configId: string
        configParameters:
        - key: string
          value: string
      deviceId: string
      licenseLevel: string
      licenseType: string
      topOfStackSerialNumber: string
    fileServiceId: string
    imageId: string
    imageUrl: string
    populateInventory: true
    projectId: string
    workflowId: string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "jsonArrayResponse": [
        {}
      ],
      "jsonResponse": {},
      "message": "string",
      "statusCode": 0
    }
"""



#!/usr/bin/python
# coding: utf-8 -*-

# (c) 2019, John Westcott IV <john.westcott.iv@redhat.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1', 'status': ['preview'], 'supported_by': 'community'}


DOCUMENTATION = '''
---
module: subscriptions
author: "John Westcott IV (@john-westcott-iv)"
short_description: Get subscription list
description:
    - Get subscriptions available to Automation Platform Controller. See
      U(https://www.ansible.com/tower) for an overview.
options:
    username:
      description:
        - Red Hat or Red Hat Satellite username to get available subscriptions.
        - The credentials you use will be stored for future use in retrieving renewal or expanded subscriptions
      required: True
      type: str
    password:
      description:
        - Red Hat or Red Hat Satellite password to get available subscriptions.
        - The credentials you use will be stored for future use in retrieving renewal or expanded subscriptions
      required: True
      type: str
    filters:
      description:
        - Client side filters to apply to the subscriptions.
        - For any entries in this dict, if there is a corresponding entry in the subscription it must contain the value from this dict
        - Note This is a client side search, not an API side search
      required: False
      type: dict
      default: {}
extends_documentation_fragment: awx.awx.auth
'''

RETURN = '''
subscriptions:
    description: dictionary containing information about the subscriptions
    returned: If login succeeded
    type: dict
'''

EXAMPLES = '''
- name: Get subscriptions
  subscriptions:
    username: "my_username"
    password: "My Password"

- name: Get subscriptions with a filter
  subscriptions:
    username: "my_username"
    password: "My Password"
    filters:
      product_name: "Red Hat Ansible Automation Platform"
      support_level: "Self-Support"
'''

from ..module_utils.controller_api import ControllerAPIModule


def main():

    module = ControllerAPIModule(
        argument_spec=dict(
            username=dict(type='str', required=True),
            password=dict(type='str', no_log=True, required=True),
            filters=dict(type='dict', required=False, default={}),
        ),
    )

    json_output = {'changed': False}

    # Check if Tower is already licensed
    post_data = {
        'subscriptions_password': module.params.get('password'),
        'subscriptions_username': module.params.get('username'),
    }
    all_subscriptions = module.post_endpoint('config/subscriptions', data=post_data)['json']
    json_output['subscriptions'] = []
    for subscription in all_subscriptions:
        add = True
        for key in module.params.get('filters').keys():
            if subscription.get(key, None) and module.params.get('filters')[key] not in subscription.get(key):
                add = False
        if add:
            json_output['subscriptions'].append(subscription)

    module.exit_json(**json_output)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: maps_import_status_v1_info
short_description: Information module for Maps Import Status V1
description:
- Get all Maps Import Status V1.
- >
   Gets the status of a map archive import operation. For a map archive import that has just been initiated, will
   provide the result of validation of the archive and a pre-import preview of what will be performed if the import
   is performed. Once an import is requested to be performed, this API will give the status of the import and upon
   completion a post-import summary of what was performed by the operation.
version_added: '6.14.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  importContextUuid:
    description:
    - >
      ImportContextUuid path parameter. The unique import context UUID given by a previous and recent call to
      maps/import/start API.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Sites ImportMapArchiveImportStatusV1
  description: Complete reference of the ImportMapArchiveImportStatusV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!import-map-archive-import-status
notes:
  - SDK Method used are
    sites.Sites.import_map_archive_import_status_v1,

  - Paths used are
    get /dna/intent/api/v1/maps/import/{importContextUuid}/status,

"""

EXAMPLES = r"""
- name: Get all Maps Import Status V1
  cisco.dnac.maps_import_status_v1_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    importContextUuid: string
  register: result

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "auditLog": {
        "children": [
          "string"
        ],
        "entitiesCount": [
          {
            "key": 0
          }
        ],
        "entityName": "string",
        "entityType": "string",
        "errorEntitiesCount": [
          {
            "key": 0
          }
        ],
        "errors": [
          {
            "message": "string"
          }
        ],
        "infos": [
          {
            "message": "string"
          }
        ],
        "matchingEntitiesCount": [
          {
            "key": 0
          }
        ],
        "subTasksRootTaskId": "string",
        "successfullyImportedFloors": [
          "string"
        ],
        "warnings": [
          {
            "message": "string"
          }
        ]
      },
      "status": "string",
      "uuid": {
        "leastSignificantBits": 0,
        "mostSignificantBits": 0
      }
    }
"""
